extends RefCounted
class_name ConservativeRasterizationExample


var _shader : RID
var _pipeline : RID

var _fb_format : int

func render(conservative_raster: bool) -> ImageTexture:
	var rd : RenderingDevice = RenderingServer.create_local_rendering_device()
	
	var shader_file: RDShaderFile = preload("res://render_simple.glsl")
	var shader_spirv: RDShaderSPIRV = shader_file.get_spirv()
	_shader = rd.shader_create_from_spirv(shader_spirv, "render_simple")
	
	var color_attachment := RDAttachmentFormat.new()
	color_attachment.usage_flags = RenderingDevice.TEXTURE_USAGE_COLOR_ATTACHMENT_BIT | RenderingDevice.TEXTURE_USAGE_CAN_COPY_FROM_BIT
	color_attachment.format = RenderingDevice.DATA_FORMAT_R8G8B8A8_SRGB
	
	_fb_format = rd.framebuffer_format_create([color_attachment])
	
	var raster_state := RDPipelineRasterizationState.new()
	raster_state.enable_conservative_rasterization = conservative_raster
	
	var color_blend_attachment := RDPipelineColorBlendStateAttachment.new()
	var color_blend_state := RDPipelineColorBlendState.new()
	color_blend_state.attachments = [color_blend_attachment]
	
	_pipeline = rd.render_pipeline_create(_shader, _fb_format, -1, RenderingDevice.RENDER_PRIMITIVE_TRIANGLES, raster_state, RDPipelineMultisampleState.new(), RDPipelineDepthStencilState.new(), color_blend_state)
	
	var texture_format := RDTextureFormat.new()
	texture_format.format = color_attachment.format
	texture_format.width = 128
	texture_format.height = 128
	texture_format.usage_bits = color_attachment.usage_flags
	
	var texture := rd.texture_create(texture_format, RDTextureView.new())
	
	var fb = rd.framebuffer_create([texture], _fb_format)
	
	var d := rd.draw_list_begin(fb, RenderingDevice.INITIAL_ACTION_CLEAR, RenderingDevice.FINAL_ACTION_READ, RenderingDevice.INITIAL_ACTION_CLEAR, RenderingDevice.FINAL_ACTION_DISCARD, PackedColorArray([Color(Color.BLACK, 0.0)]))
	rd.draw_list_bind_render_pipeline(d, _pipeline)
	rd.draw_list_draw(d, false, 1, 3)
	rd.draw_list_end()
	
	rd.submit()
	rd.sync()
	
	var data := rd.texture_get_data(texture, 0)
	
	var image := Image.new()
	image.set_data(texture_format.width, texture_format.height, false, Image.FORMAT_RGBA8, data)
	
	return ImageTexture.create_from_image(image)
